#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2util.h>
#include <sps2regstructs.h>
#include <signal.h>
#include "PS2Defines.h"
#include "sps2wrap.h"
#include "dma.h"
#include "texture.h"
#include "pad.h"
#include "game.h"

bool CGame::m_bRunning;

CGame::CGame(void)
{
	m_bRunning = true;
	GameState = SPLASH;
}

CGame::~CGame(void)
{
	// Nothing
}

bool CGame::Start()
{		
	//set up our manager and the memory
	SPS2Manager.Initialise(512);	// 4096 * 4K Pages = 16MB Total

	VIFStaticDMA.Initialise(256);	// 1536 * 4K Pages = 6MB Static DMA
	VIFDynamicDMA.Initialise(128);	// 256 * 4K Pages * 2 Buffers =
									// 2MB Dynamic DMA
									
	// Capture all signals
	for(int sig=0; sig<128; sig++) 
		signal(sig, CGame::SignalHandler);
									
	// Initialise control pad 0
	if(!pad_init(PAD_0, PAD_INIT_LOCK | PAD_INIT_ANALOGUE | PAD_INIT_PRESSURE))
	{
		printf("Failed to initialise control pad\n");
		pad_cleanup(PAD_0);
		return false;
	}

	// Set up the DMA packet to clear the screen. We want to clear to blue.
	SPS2Manager.InitScreenClear(0, 0x05, 0x10);
	
	// Load the font bitmap and data
	if(!Font.Load("font.dat", true))
	{
		printf("Can't load font.dat\n");
		return false;
	}
	if(!FontTex.LoadBitmap("font.bmp", false, true))
	{
		printf("Can't load font.bmp\n");
		return false;
	}

	// Upload Font texture to VRAM page 496
	FontTex.Upload(TEXBUF496);
	
	// Load the game bitmap
	if(!Game.LoadBitmap("game.bmp", true, false))
	{
		printf("Can't load Texture\n");
		return false;
	}
	
	// Upload the game bitmap to VRAM page 480
	Game.Upload(TEXBUF480);
	
	return true;
}

void CGame::GameLoop()
{
	pad_update(PAD_0);
	
	// Check for immediate exit condition
	// Use this for debug purposes
	if((pad[0].buttons & PAD_START)&&(pad[0].buttons & PAD_SELECT)) 
		SetRunningStatus(false);
	
	
	switch (GameState)
	{
		case SPLASH:
			GameSplash();
			break;
			
		case MENU:
			GameMenu();
			break;
			
		case LEVEL1:
			GameLevel1();
			break;
			
		case EXIT:
			GameExit();
			break;
			
		default:
			printf("Undefined Game State\n");
			SetRunningStatus(false);
			break;
	}
	
	
}

void CGame::GameSplash(void)
{
	static int FrameCounter = 0;
	const int NumFrames = 250;
	
	if(FrameCounter++ == NumFrames)GameState = MENU;
	
	VIFDynamicDMA.Fire();
	
	SPS2Manager.BeginScene();
	FontTex.Select();
	Font.printfC(  200, -240, 127, 127, 127, 127, "Frame: %d" , FrameCounter);
	Font.printfC(  0, 0, 127, 127, 127, 127, "Put Splash Screen Here");
	
	SPS2Manager.EndScene();			
}

void CGame::GameMenu(void)
{
	static int FrameCounter = 0;
	FrameCounter++;
	
	VIFDynamicDMA.Fire();
	
	SPS2Manager.BeginScene();
	
	FontTex.Select();
	Font.printfC(  200, -240, 127, 127, 127, 127, "Frame: %d" , FrameCounter);
	Font.printfC(  0, -100, 127, 127, 127, 127, "Make Selection:");
	Font.printfC(  0, -50, 127, 127, 127, 127, "CROSS to start");
	Font.printfC(  0, -25, 127, 127, 127, 127, "SELECT to exit");
	
	SPS2Manager.EndScene();	
	
	if(pad[0].pressed & PAD_CROSS)GameState = LEVEL1;
	if(pad[0].pressed & PAD_SELECT)GameState = EXIT;
			
}



void CGame::GameLevel1(void)
{
	static int FrameCounter = 0;
	FrameCounter++;
	
	VIFDynamicDMA.Fire();
	
	SPS2Manager.BeginScene();
	
	FontTex.Select();
	Font.printfC(  200, -240, 127, 127, 127, 127, "Frame: %d" , FrameCounter);
	
	Game.Select();
	Sprite.Rotate(0.01);
	Sprite.Move();
	Sprite.Render();
	
	SPS2Manager.EndScene();	
	
	if(pad[0].pressed & PAD_SELECT)GameState = EXIT;		
}


void CGame::GameExit(void)
{
	static int FrameCounter = 0;
	const int NumFrames = 250;
	
	if(FrameCounter++ == NumFrames)SetRunningStatus(false);
	
	VIFDynamicDMA.Fire();
	
	SPS2Manager.BeginScene();
	
	FontTex.Select();
	Font.printfC(  200, -240, 127, 127, 127, 127, "Frame: %d" , FrameCounter);
	Font.printfC(  0, -30, 127, 127, 127, 127, "GOOD Game\n   BYE");
	
	SPS2Manager.EndScene();			
}



void CGame::Stop()
{
	pad_cleanup(PAD_0);
}


void CGame::SignalHandler(int Signal)
{
	SetRunningStatus(false);
}
	
	